#include <msp430.h>
#include "defines.h"
;=----------------------------------------=
; Xmit: send data to WS2812 LEDs
; entry: r15 : first node of list with LED data.
;
; return: -
;
; uses:         r4:  raw byte to send
;		r5:  bit counter
;		r6:  '0' bit pattern
;		r7:  '1' bit pattern (these save 2? cycles per loop)
;		r8:  node ptr (copy of r15)
;		r9:  byte counter
; 		r10: total LEDs to xmit
;		r11: led bit counter (r,g,b)
; 		r12: LED_PIN
;=----------------------------------------=
	;; @16MHz
	;; T0H = 7 cycles
	;; T0L = 13 cycles
	;; T1H = 13 cycles
	;; T1L = 7 cycles

.global xmitData
xmitData:
	dint	
	push.w 	r12	;hi
	push.w	r11	;lo
	push.w 	r10
	push.w  r9
	push.w	r7	;bit count
	push.w  r6	;n2
	push.w  r5	;n1
	push.w  r4
	mov.w 	#0,r9
	
	mov.b	&__P1OUT, r12
	bis.b	#LED_PIN, r12	;set hi
	mov.b	&__P1OUT, r11
	bic.b	#LED_PIN, r11	;set lo
	
	
	mov.w 	#(NUM_LEDS*3)+1,r10
	jmp 	compare


XmitLoop:
	mov	#4,r7		;4*2bits	(2)
	;; BIT 7
	mov.b	r11,r5		;n1=lo		(1)
	rla.b	r4		;test msb bit   (1)
	jnc	zero7		;		(2)
	mov.b	r12,r5		;n1=hi		(1)
zero7:	
	mov.b	r12,&__P1OUT	;output hi	(3)	*
	mov.b 	r11,r6		;n2=lo		(1)
	rla.b	r4		;test Bit 6	(1)
//	nop			;		(1)
//	nop			;		(1)	
	mov.b	r5,&__P1OUT	;output=n1	(3)	*		
	jnc	zero6		;		(2)
	mov	r12,r6		;n2=hi		(1)
zero6:
	mov.b	r11,&__P1OUT	;output=lo	(3)	*
	nop			;		(4)
	nop
	nop
	nop
	;; BIT6
	mov.b	r12,&__P1OUT	;output hi	(3)	*
	mov.b	r11,r5		;n1=lo		(1)
	rla.b	r4		;test bit 5	(1)
//	nop			;		(1)
//	nop			;		(1)
	mov.b	r6,&__P1OUT	;output=n2	(3)	*
	jnc	zero5		;		(2)
	mov	r12,r5		;n1=hi		(1)
zero5:
	mov.b	r11,&__P1OUT	;output=lo	(3)	*
	dec	r7		;		(2)
	jnz	zero7		;		(2)
	
	
compare:
	mov.b 	@r15+,r4	; (2)
	dec	r10		; (2)
	jnz 	XmitLoop	; (2)

	
	mov.w	#100,r9
reset:
	dec.w 	r9
	jnz	reset
	
	pop.w  r4
	pop.w  r5
	pop.w  r6
	pop.w  r7
	pop.w  r9
	pop.w  r10
	pop.w  r11
	pop.w  r12
	eint
	ret

