/*
Mobile lumineux main.c
 */



#include <msp430.h>
#include <stdint.h>

#include "defines.h"
#include "colour.h"
#include "ledBuffer.h"
#include "dco.h"
#include "led.h"

#define NB_ANIM 6         // Number of animations
#define DELAI_MENU 60     // Menu delay
            
int animNum=0;            // Animation number
volatile int btnTime;     // Button debounce
volatile int menuTime;    // Menu timeout
hsl_t animCol[NUM_LEDS];  //Colors used in anim

// Animation functions
void animFade(void); 
void animTrans(void);
void animRand(void);
void animAltern(void);
void animSeq(void);
void animVeilleuse(void);

// Pointers to animation functions
void (*animFctPtrs[NB_ANIM])(void)={&animFade,&animTrans,&animRand,
									&animAltern,&animSeq,&animVeilleuse};

// Pointer to current animation function
void (*fctAnim)(void);

void clrAnimCom(){
  int i;
  char* p = (char*)animCol;
  for( i = 0; i < sizeof(animCol); i++)
    *p++ = 0;
}

// Menu to select animation mode
void menu(void){
  menuTime=DELAI_MENU;
  hsl_t Acolour = {.h = 0, .s = 255, .l =200};
  while(menuTime){
	  if(!(BTN_IN & BTN_PIN) && (btnTime==0)){
		  clrAnimCom();
		  if(++animNum==NB_ANIM) animNum=0;
		  fctAnim = animFctPtrs[animNum];
		  clrData();
		  Acolour.h=(animNum<<5)+(animNum<<3);
		  hsl2rgb(&Acolour,&ledBuffer[ledOrder[animNum]]);
		  hsl2rgb(&Acolour,&ledBuffer[ledOrder[NUM_LEDS-1]]);
		  xmitData((char*)ledBuffer);
		  btnTime=10;
		  menuTime=DELAI_MENU;
	  }
  }
  clrPixel(animNum);
  clrData();
}


#define VEILLE_LUM 3;
#define VEILLE_TMR 300;
void animVeilleuse(){
	static int veille_tmr=1;
	int i;
	if(--veille_tmr==0){
		veille_tmr = VEILLE_TMR;
		animCol[0].h = rand() & 0xFF;
		animCol[0].l = VEILLE_LUM;
		animCol[0].s = 0xFF;
		for(i=0;i<NUM_LEDS;i++)
			hsl2rgb(animCol,&ledBuffer[i]);
		xmitData((char*)ledBuffer);
	}
}


// Light all the led sequentially
#define SPEED_SEQ 4
#define MAX_FADE_SEQ 100

#define STATE_INC 0
#define STATE_DEC 1
void animSeq(){
	static int nbLeds = 0;
	static int state = STATE_INC;
	
	if(animCol[nbLeds].l>MAX_FADE_SEQ){
		nbLeds++;
		if(nbLeds==NUM_LEDS){
			state = STATE_DEC;
		}
		else{
			animCol[nbLeds].h = rand()&0xFF;   //rand()%254
			animCol[nbLeds].l = 0;
			animCol[nbLeds].s = 255;
		}
	}
	int i;
	if(state == STATE_INC){
		animCol[nbLeds].l += SPEED_SEQ;
		hsl2rgb(&animCol[nbLeds],&ledBuffer[ledOrder[nbLeds]]);
	} 
	else if (state==STATE_DEC){
		for(i=0;i<NUM_LEDS;i++){
			animCol[i].l-=SPEED_SEQ;
			hsl2rgb(&animCol[i],&ledBuffer[ledOrder[i]]);
		}
		if(animCol[0].l<SPEED_SEQ){
			state = STATE_INC;
			nbLeds = 0;
			animCol[0].h = rand()&0xFF;   //rand()%254
			animCol[0].l = 0;
			animCol[0].s = 255;
		}
	}
	xmitData((char*)ledBuffer);
}


// Alternate between objects and center
#define STATE_POMPON1 0
#define STATE_PAP 1
#define STATE_POMPON2 2
#define STATE_FLEUR 3

#define SPEED_ALT 4
#define FADE_MAX 200
void animAltern(){
  static int delta=SPEED_ALT;
  static int state=STATE_POMPON1;
  switch(state){
  case STATE_POMPON1:
  case STATE_POMPON2:
	  hsl2rgb(animCol,&ledBuffer[POMPON_LED]);
	  break;
  case STATE_PAP:
	  hsl2rgb(animCol,&ledBuffer[PAP1_LED1]);
	  hsl2rgb(animCol,&ledBuffer[PAP1_LED2]);
	  hsl2rgb(animCol,&ledBuffer[PAP2_LED1]);
	  hsl2rgb(animCol,&ledBuffer[PAP2_LED2]);
	  break;
  case STATE_FLEUR:
	  hsl2rgb(animCol,&ledBuffer[FLEUR1_LED]);
	  hsl2rgb(animCol,&ledBuffer[FLEUR2_LED]);
  }
  animCol[0].l+=delta;
  if(animCol[0].l>FADE_MAX){
    delta=-SPEED_ALT;
  }
  if(animCol[0].l<SPEED_ALT){
    delta=SPEED_ALT;
    if(state++==3)
	  state=0;
	animCol[0].h=rand()&0xFF; //rand()%254
	animCol[0].s=0xFF; 
	clrData();
  }
  xmitData((char*)ledBuffer);
}

// Fade each led in and out with a constant colour
#define FADE_LUM 200
void animFade(){
  static int delta=1;
  uint16_t tempH = animCol[0].h;
  animCol[0].s=255;
  //animCol[0].l=FADE_LUM;
  int i;
  for(i=0;i<NUM_LEDS;i++){
    hsl2rgb(animCol,&ledBuffer[i]);
    //tempHSL.h+=30;
    animCol[0].h+=30;
  }
  animCol[0].l+=delta;
  if(animCol[0].l>200){
    delta=-1;
  }
  if(animCol[0].l<20){
    delta=1;
  }
  animCol[0].h = tempH;
  xmitData((char*)ledBuffer);
}

// Change the colour of each led continuously
#define TRANS_LUM 200
void animTrans(){
  uint16_t tempH = animCol[0].h;
  animCol[0].s=255;
  animCol[0].l=TRANS_LUM;
  int i;
  for(i=0;i<NUM_LEDS;i++){
    hsl2rgb(animCol,&ledBuffer[i]);
    animCol[0].h+=30;
  }
  animCol[0].h = tempH+1;
  xmitData((char*)ledBuffer);
}


// Light up leds randomly with a random colour
#define SPEED2 6
#define MAXLUM 200
#define DELAY2 15
void animRand(){
  static int delta[NUM_LEDS];
  static int timer=DELAY2;

  // time to light up a new led
  if(--timer==0){
    timer = DELAY2;
    int numled = rand()&0x7;//%NUM_LEDS;
	if(numled==7) numled=0;
    // choosen LED is ready
    if(animCol[numled].l==0){
      delta[numled] = SPEED2;
      animCol[numled].l=SPEED2;
      animCol[numled].h=rand()&0xFF;  //rand()%254
      animCol[numled].s=255;
    }
  }

  // Process the leds
  int i;
  for(i=0;i<NUM_LEDS;i++){
    if(animCol[i].l){              // Test if led is lit
      if(animCol[i].l<SPEED2){
		  clrPixel(i);
      }
      if(animCol[i].l>MAXLUM)
		  delta[i]=-SPEED2;
      animCol[i].l+=delta[i];
      hsl2rgb(&animCol[i],&ledBuffer[i]);
    }
  }
  xmitData((char*)ledBuffer);
}





int main(void)
{
  WDTCTL = WDTPW + WDTHOLD;                   // Stop WDT interrupt
	
  P2SEL |= BIT7 | BIT6; // ext osc
  
  LED_DIR |= LED_PIN;
  LED_OUT &= ~LED_PIN;
	
  BTN_DIR &= ~BTN_PIN;
  //BCSCTL1 = CALBC1_16MHZ;          //run at 8Mhz
  //DCOCTL = CALDCO_16MHZ;

  BCSCTL1 &= ~DIVA_3;  //DIVA_0 
  BCSCTL2 |= DIVM_0;
  BCSCTL3 = XCAP_2;    //~10pF

  Set_DCO(DELTA_16MHZ); //16 MHz
    
  TACCTL1 = CCIE;                             // CCR0 interrupt enabled
  
  TACTL = TASSEL_1 + MC_2 + TACLR;           // ACLK, cont-mode

  fctAnim = animTrans;
  

  while(1){
    if(!(BTN_IN & BTN_PIN) && (btnTime==0)){
      menu();     //Button pressed, goto menu
    }
    // Animate
    fctAnim(); 
    LPM0;
  }
}

// TimerA1 CCR1/Overflow Interrupt
__attribute__((interrupt(TIMER0_A1_VECTOR))) void TimerA(void)
{
  // CCR1
  TACCTL1 &= ~CCIFG;
  TACCR1 += 1000;

  LPM0_EXIT;
  if(btnTime)
    btnTime--;
  if(menuTime)
    menuTime--;
}

